/****************************************************************************
** Generated QML type registration code
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <QtQml/qqml.h>
#include <QtQml/qqmlmoduleregistration.h>

#if __has_include(<private/qblendanimationnode_p.h>)
#  include <private/qblendanimationnode_p.h>
#endif
#if __has_include(<private/qblendtreenode_p.h>)
#  include <private/qblendtreenode_p.h>
#endif
#if __has_include(<private/qtimelineanimationnode_p.h>)
#  include <private/qtimelineanimationnode_p.h>
#endif


#if !defined(QT_STATIC)
#define Q_QMLTYPE_EXPORT Q_DECL_EXPORT
#else
#define Q_QMLTYPE_EXPORT
#endif
Q_QMLTYPE_EXPORT void qml_register_types_QtQuick_Timeline_BlendTrees()
{
    qmlRegisterModule("QtQuick.Timeline.BlendTrees", 6, 0);
    QT_WARNING_PUSH QT_WARNING_DISABLE_DEPRECATED
    qmlRegisterTypesAndRevisions<QBlendAnimationNode>("QtQuick.Timeline.BlendTrees", 6);
    qmlRegisterTypesAndRevisions<QBlendTreeNode>("QtQuick.Timeline.BlendTrees", 6);
    qmlRegisterTypesAndRevisions<QTimelineAnimationNode>("QtQuick.Timeline.BlendTrees", 6);
    QT_WARNING_POP
    qmlRegisterModule("QtQuick.Timeline.BlendTrees", 6, 12);
}

static const QQmlModuleRegistration qtQuickTimelineBlendTreesRegistration("QtQuick.Timeline.BlendTrees", qml_register_types_QtQuick_Timeline_BlendTrees);
